package com.fjhx.sale.flow;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.fjhx.common.entity.InOutBo;
import com.fjhx.common.enums.FlowStatusEnum1;
import com.fjhx.common.enums.InOutType;
import com.fjhx.common.utils.Assert;
import com.fjhx.common.utils.Utils;
import com.fjhx.flow.core.FlowDelegate;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.fjhx.item.enums.ProductAvailableRecordType;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.purchase.entity.purchase.enums.PurchaseDataResourceEnum;
import com.fjhx.sale.entity.purchase.dto.EhsdPurchaseDto;
import com.fjhx.sale.entity.purchase.po.EhsdPurchase;
import com.fjhx.sale.entity.purchase.po.EhsdPurchaseProduct;
import com.fjhx.sale.entity.purchase.po.EhsdPurchaseProject;
import com.fjhx.sale.service.purchase.EhsdPurchaseProductService;
import com.fjhx.sale.service.purchase.EhsdPurchaseProjectService;
import com.fjhx.sale.service.purchase.EhsdPurchaseService;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * 采购变更流程
 */
@Component
public class EhsdPurchaseUpdateFlow extends FlowDelegate {
    @Autowired
    private EhsdPurchaseService ehsdPurchaseService;
    @Autowired
    private EhsdPurchaseFlow ehsdPurchaseFlow;
    @Autowired
    private EhsdPurchaseProductService ehsdPurchaseProductService;
    @Autowired
    private EhsdPurchaseProjectService ehsdPurchaseProjectService;
    @Autowired
    private ProductInfoService productInfoService;

    @Override
    public String getFlowKey() {
        return "purchase_update_flow";
    }

    /**
     * 发起流程
     */
    @Override
    public Long start(Long flowId, JSONObject submitData) {
        EhsdPurchaseDto purchase = submitData.toJavaObject(EhsdPurchaseDto.class);

        //清理Id防止前端误传
        purchase.setId(null);

        //赋值流程id
        purchase.setFlowId(flowId);

        Long oldPurchaseId = purchase.getOldPurchaseId();
        Assert.notEmpty(oldPurchaseId, "原采购订单id不能为空");

        //判断合同是否在变更中，防止用户缓存导致重复变更
        EhsdPurchase oldPurchase = ehsdPurchaseService.getById(oldPurchaseId);
        Assert.notEmpty(oldPurchase, "查询不到原采购订单信息");
        if (oldPurchase.getStatus().equals(FlowStatusEnum1.UPDATE_LOADING.getKey())) {
            throw new ServiceException("原采购订单正在变更中，禁止重复变更！");
        }

        //原采购订单修改为变更中
        ehsdPurchaseService.update(q -> q.eq(EhsdPurchase::getId, oldPurchaseId).set(EhsdPurchase::getStatus, FlowStatusEnum1.UPDATE_LOADING.getKey()));

        //赋值新单号
        purchase.setCode(Utils.getNewCode(oldPurchase.getCode()));

        //获取明细信息
        List<EhsdPurchaseProduct> purchaseProductList = purchase.getPurchaseProductList();
        List<EhsdPurchaseProject> purchaseProjectList = purchase.getPurchaseProjectList();
        //清空物料，收费项目id 防止前端误传
        purchaseProductList.forEach(item -> item.setId(null));
        purchaseProjectList.forEach(item -> item.setId(null));

        //调用采购发起公共代码块
        purchase = ehsdPurchaseFlow.connStart(purchase);

        return purchase.getId();
    }

    /**
     * 结束流程
     */
    @Override
    public void end(Long flowId, Long businessId, JSONObject submitData) {
        //通过业务ID查询采购数据
        EhsdPurchase purchase = ehsdPurchaseService.getById(businessId);
        Assert.notEmpty(purchase, "采购单不存在,或已被删除");

        //原合同改为 已变更
        EhsdPurchase oldEhsdPurchase = ehsdPurchaseService.getById(purchase.getOldPurchaseId());
        oldEhsdPurchase.setStatus(FlowStatusEnum1.UPDATE.getKey());
        ehsdPurchaseService.updateById(oldEhsdPurchase);

        //修改可用库存
        //回滚旧合同数据
        List<InOutBo> oldInOutBoList = new ArrayList<>();
        List<EhsdPurchaseProduct> oldPurchaseProductList = ehsdPurchaseProductService.list(q -> q.eq(EhsdPurchaseProduct::getPurchaseId, purchase.getOldPurchaseId()));
        for (EhsdPurchaseProduct purchaseProduct : oldPurchaseProductList) {
            //如果数据来源是申购 操作可用库存(可用库存 = 当前可用库存 - 采购明细数量)
            if (Objects.equals(purchaseProduct.getDataResource(), PurchaseDataResourceEnum.DATA_RESOURCE_0)) {
                InOutBo inOutBo = new InOutBo();
                inOutBo.setProductId(purchaseProduct.getProductId());
                inOutBo.setQuantity(purchaseProduct.getQuantity());
                oldInOutBoList.add(inOutBo);
            }
        }
        productInfoService.editAvailableQuantity(oldInOutBoList, InOutType.OUT, purchase.getOldPurchaseId(), ProductAvailableRecordType.PURCHASE_UPDATE, purchase.getCompanyId());


        //将可用库存改为新合同数据
        List<InOutBo> inOutBoList = new ArrayList<>();
        List<EhsdPurchaseProduct> purchaseProductList = ehsdPurchaseProductService.list(q -> q.eq(EhsdPurchaseProduct::getPurchaseId, businessId));
        for (EhsdPurchaseProduct purchaseProduct : purchaseProductList) {
            //如果数据来源是申购 操作可用库存(可用库存 = 当前可用库存 + 采购明细数量)
            if (Objects.equals(purchaseProduct.getDataResource(), PurchaseDataResourceEnum.DATA_RESOURCE_0)) {
                InOutBo inOutBo = new InOutBo();
                inOutBo.setProductId(purchaseProduct.getProductId());
                inOutBo.setQuantity(purchaseProduct.getQuantity());
                inOutBoList.add(inOutBo);
            }
        }
        productInfoService.editAvailableQuantity(inOutBoList, InOutType.IN, businessId, ProductAvailableRecordType.PURCHASE_UPDATE, purchase.getCompanyId());


        //调用采购发起流程的结束方法
        ehsdPurchaseFlow.end(flowId, businessId, submitData);

        Long oldPurchaseId = purchase.getOldPurchaseId();

        //先将当前合同id 以及 当前合同关联数据的合同id 改为临时id
        long tempId = IdWorker.getId();
        ehsdPurchaseService.updatePurchaseId(businessId, tempId, SecurityUtils.getUserId(), null);
        ehsdPurchaseProductService.update(q -> q.eq(EhsdPurchaseProduct::getPurchaseId, businessId).set(EhsdPurchaseProduct::getPurchaseId, tempId));
        ehsdPurchaseProjectService.update(q -> q.eq(EhsdPurchaseProject::getPurchaseId, businessId).set(EhsdPurchaseProject::getPurchaseId, tempId));


        //将旧合同的id改为新合同id
        ehsdPurchaseService.updatePurchaseId(oldPurchaseId, businessId, SecurityUtils.getUserId(), null);
        ehsdPurchaseProductService.update(q -> q
                .eq(EhsdPurchaseProduct::getPurchaseId, oldPurchaseId)
                .set(EhsdPurchaseProduct::getPurchaseId, businessId)
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
        ehsdPurchaseProjectService.update(q -> q
                .eq(EhsdPurchaseProject::getPurchaseId, oldPurchaseId)
                .set(EhsdPurchaseProject::getPurchaseId, businessId)
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
        //将新合同id改为旧合同id
        ehsdPurchaseService.updatePurchaseId(tempId, oldPurchaseId, SecurityUtils.getUserId(), businessId);
        ehsdPurchaseProductService.update(q -> q
                .eq(EhsdPurchaseProduct::getPurchaseId, tempId)
                .set(EhsdPurchaseProduct::getPurchaseId, oldPurchaseId)
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
        ehsdPurchaseProjectService.update(q -> q
                .eq(EhsdPurchaseProject::getPurchaseId, tempId)
                .set(EhsdPurchaseProject::getPurchaseId, oldPurchaseId)
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    /**
     * 重新发起
     */
    @Override
    public void relaunch(Long flowId, Long businessId, FlowStatusEnum flowStatus, JSONObject submitData) {
        EhsdPurchaseDto purchase = submitData.toJavaObject(EhsdPurchaseDto.class);
        if (ObjectUtils.isEmpty(purchase)) {
            throw new ServiceException("采购数据不能为空");
        }
        purchase.setFlowId(flowId);
        ehsdPurchaseFlow.connStart(purchase);
    }

    /**
     * 驳回
     */
    @Override
    public void reject(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        ehsdPurchaseService.update(q -> q
                .eq(EhsdPurchase::getId, businessId)
                .set(EhsdPurchase::getStatus, FlowStatusEnum1.REJECT.getKey())
                .set(EhsdPurchase::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );

        //恢复原合同
        EhsdPurchase ehsdPurchase = ehsdPurchaseService.getById(businessId);
        ehsdPurchaseService.update(q -> q
                .eq(BasePo::getId, ehsdPurchase.getOldPurchaseId())
                .set(EhsdPurchase::getStatus, FlowStatusEnum1.PASS.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    /**
     * 作废
     */
    @Override
    public void cancellation(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        super.cancellation(flowId, businessId, flowStatus);
        ehsdPurchaseService.update(q -> q
                .eq(EhsdPurchase::getId, businessId)
                .set(EhsdPurchase::getStatus, FlowStatusEnum1.CANCELLATION.getKey())
                .set(EhsdPurchase::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );

        //恢复原合同
        EhsdPurchase ehsdPurchase = ehsdPurchaseService.getById(businessId);
        ehsdPurchaseService.update(q -> q
                .eq(BasePo::getId, ehsdPurchase.getOldPurchaseId())
                .set(EhsdPurchase::getStatus, FlowStatusEnum1.PASS.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

}
